ChatSounds = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

function ChatSounds:OnInitialize()

	self:RegisterEvent("CHAT_MSG_CHANNEL")
	
	self:RegisterEvent("CHAT_MSG_GUILD", "GroupChat")
	self:RegisterEvent("CHAT_MSG_OFFICER", "GroupChat")
	self:RegisterEvent("CHAT_MSG_PARTY", "GroupChat")
	self:RegisterEvent("CHAT_MSG_RAID", "GroupChat")
	self:RegisterEvent("CHAT_MSG_RAID_LEADER", "GroupChat")
	self:RegisterEvent("CHAT_MSG_BATTLEGROUND", "GroupChat")
	self:RegisterEvent("CHAT_MSG_BATTLEGROUND_LEADER", "GroupChat")
	
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM")

	self.sounds = {
		["TellMessage"] = "TellMessage",
		["Heart"] = "Interface\\AddOns\\ChatSounds\\Sounds\\Heart.wav",
		["Kachink"] = "Interface\\AddOns\\ChatSounds\\Sounds\\Kachink.wav",
		["Link"] = "Interface\\AddOns\\ChatSounds\\Sounds\\Link.wav",
		["Text1"] = "Interface\\AddOns\\ChatSounds\\Sounds\\Text1.wav",
		["Text2"] = "Interface\\AddOns\\ChatSounds\\Sounds\\Text2.wav",
	}
	
	self:RegisterDB("ChatSoundsDB")
	self:RegisterDefaults("profile", {
		["incoming"] = {
			["GUILD"] = "Kachink",
			["OFFICER"] = "Link",
			["PARTY"] = "Text1",
			["RAID"] = "Text2",
			["WHISPER"] = "Heart",
		},
		["outgoing"] = {
			["GUILD"] = "none",
			["OFFICER"] = "none",
			["PARTY"] = "none",
			["RAID"] = "none",
			["WHISPER"] = "none",
		},
		["customlist"] = {
			"healer",
			"pall",
			"priest",
			"warrior",
			"mage",
			"hunter",
			"druid",
			"lock",
			"rogue",
		},
	} )
	self:RegisterChatCommand({ "/chatsounds"}, {
		type = "execute", name = "cs", desc = "cs",
		func = function() self:Options() end,
	})
end

-- sounds for custom channels
function ChatSounds:CHAT_MSG_CHANNEL()
	if arg2 ~= UnitName("player") then
		for _,value in pairs(self.db.profile.customlist) do
			if string.find(string.lower(arg9), string.lower(value)) then
				self:PlaySound(self.db.profile.incoming["OFFICER"])
			end
		end
	end	
end

-- sounds for party, raid (leader and bg included), guild, and officer
function ChatSounds:GroupChat()
	local msgtype = string.sub(event, 10)
	if arg2 == UnitName ("player") then
		if msgtype == "RAID_LEADER" or msgtype == "BATTLEGROUND" or msgtype == "BATTLEGROUND_LEADER" then
			msgtype = "RAID"
		end
		self:PlaySound(self.db.profile.outgoing[msgtype])
	else
		if msgtype == "RAID_LEADER" or msgtype == "BATTLEGROUND" or msgtype == "BATTLEGROUND_LEADER" then
			msgtype = "RAID"
		end
		self:PlaySound(self.db.profile.incoming[msgtype])
	end
end

-- sounds for incoming whispers
function ChatSounds:CHAT_MSG_WHISPER()
	-- La Vendetta has that annoying whisper function, so don't play a sound for those
	if strsub(arg1, 1, 4) == "LVPN" or strsub(arg1, 1, 4) == "LVBM" then return end
	self:PlaySound(self.db.profile.incoming["WHISPER"])
end

-- sounds for outgoing whispers
function ChatSounds:CHAT_MSG_WHISPER_INFORM()
	if strsub(arg1, 1, 4) == "LVPN" or strsub(arg1, 1, 4) == "LVBM" then return end
	self:PlaySound(self.db.profile.outgoing["WHISPER"])
end

function ChatSounds:PlaySound(sound)
	if not sound or not self.sounds[sound] then return end
	if sound ~= "TellMessage" then
		PlaySoundFile(self.sounds[sound])
	else
		PlaySound(self.sounds[sound])
	end
end


-- OPTIONS
function ChatSounds:Options()
	if not self.initmenu then self:InitMenu() end
	self.dewdrop:Open( UIParent,
		'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		'cursorX', true, 'cursorY', true )
end

function ChatSounds:InitMenu()
	local soundslist = { "TellMessage", "Heart", "Kachink", "Link", "Text1", "Text2", "none"}
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "ChatSounds",
				order = 1,
			},
			custom = {
				type = "text",
				name = "Add a custom channel",
				desc = "Play a sound for a certain channel name (can be a substring)",
				set = function(v) tinsert(db.customlist, v) end,
				get = function() return end,
				usage = "",
				order = 4,
			},
			reset = {
				type = "execute",
				name = "Reset settings",
				desc = "Restore default settings and resets custom channel list",
				func = function() db.customlist = { } ChatSounds:ResetDB() end,
				order = 5,
			},
			inc = {
				type = "group",
				name = "Incoming Sounds",
				desc = "Sound selection for incoming chat messages",
				order = 2,
				args = {
					party = {
						type = "text",
						name = "Party",
						desc = "Sound for incoming party messages",
						get = function() return db.incoming["PARTY"] end,
						set = function(v) db.incoming["PARTY"] = v end,
						order = 1,
						validate = soundslist,
					},
					raid = {
						type = "text",
						name = "Raid",
						desc = "Sound for incoming raid or battleground group/leader messages",
						get = function() return db.incoming["RAID"] end,
						set = function(v) db.incoming["RAID"] = v end,
						order = 2,
						validate = soundslist,
					},
					guild = {
						type = "text",
						name = "Guild",
						desc = "Sound for incoming guild messages",
						get = function() return db.incoming["GUILD"] end,
						set = function(v) db.incoming["GUILD"] = v end,
						order = 3,
						validate = soundslist,
					},
					officer = {
						type = "text",
						name = "Officer/Custom",
						desc = "Sound for incoming officer or custom channel messages",
						get = function() return db.incoming["OFFICER"] end,
						set = function(v) db.incoming["OFFICER"] = v end,
						order = 4,
						validate = soundslist,
					},
					whisper = {
						type = "text",
						name = "Whisper",
						desc = "Sound for incoming whisper messages",
						get = function() return db.incoming["WHISPER"] end,
						set = function(v) db.incoming["WHISPER"] = v end,
						order = 5,
						validate = soundslist,
					},
				},
			},
			out = {
				type = "group",
				name = "Outgoing Sounds",
				desc = "Sound selection for outgoing (from you) chat messages",
				order = 3,
				args = {
					party = {
						type = "text",
						name = "Party",
						desc = "Sound for outgoing party messages",
						get = function() return db.outgoing["PARTY"] end,
						set = function(v) db.outgoing["PARTY"] = v end,
						order = 1,
						validate = soundslist,
					},
					raid = {
						type = "text",
						name = "Raid",
						desc = "Sound for outgoing raid or battleground group/leader messages",
						get = function() return db.outgoing["RAID"] end,
						set = function(v) db.outgoing["RAID"] = v end,
						order = 2,
						validate = soundslist,
					},
					guild = {
						type = "text",
						name = "Guild",
						desc = "Sound for outgoing guild messages",
						get = function() return db.outgoing["GUILD"] end,
						set = function(v) db.outgoing["GUILD"] = v end,
						order = 3,
						validate = soundslist,
					},
					officer = {
						type = "text",
						name = "Officer/Custom",
						desc = "Sound for outgoing officer or custom channel messages",
						get = function() return db.outgoing["OFFICER"] end,
						set = function(v) db.outgoing["OFFICER"] = v end,
						order = 4,
						validate = soundslist,
					},
					whisper = {
						type = "text",
						name = "Whisper",
						desc = "Sound for outgoing whisper messages",
						get = function() return db.outgoing["WHISPER"] end,
						set = function(v) db.outgoing["WHISPER"] = v end,
						order = 5,
						validate = soundslist,
					},
				},
			},
		},
	}
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self.initMenu = 1
end
